@@ -4,6 +4,7 @@ from typing import Any, List, MutableMapping
 
 import re
 import textwrap
+import warnings
 
 from diot import Diot, OrderedDiot
 from pipen.defaults import ProcInputType
@@ -18,9 +19,17 @@ __all__ = (
 )
 
 ITEM_LINE_REGEX = re.compile(
-    r"(?P<name>^[\w-]+)\s*(?:\((?P<attrs>.+?)\))?:(?P<help>.+)?$"
+    r"^(?P<name>[^\s]+)\s*(?:\((?P<attrs>.+?)\))?:(?P<help>.+)?$"
 )
-ITEM_ATTR_REGEX = re.compile(r"(?P<name>[\w-]+)(?:\s*:\s*(?P<value>.+))?")
+ITEM_ATTR_REGEX = re.compile(r"^(?P<name>[\w-]+)(?:\s*[:=]\s*(?P<value>.+))?$")
+
+
+class MalFormattedAnnotationError(Exception):
+    """Raised when the annotation is malformatted"""
+
+
+class UnknownAnnotationItemWarning(Warning):
+    """Raised when the annotation item is unknown"""
 
 
 def _is_iterable(obj: Any) -> bool:
@@ -41,7 +50,10 @@ def _dedent(lines: List[str]) -> List[str]:
     return textwrap.dedent("\n".join(lines)).splitlines()
 
 
-def _parse_terms(lines: List[str], prefix: str | None = None) -> Diot:
+def _parse_terms(
+    lines: List[str],
+    prefix: str | None = None,
+) -> Diot:
     """Parse a list of lines as terms.
     """
     terms = OrderedDiot()
@@ -49,12 +61,14 @@ def _parse_terms(lines: List[str], prefix: str | None = None) -> Diot:
     sublines = []
     item = None
     just_matched = False
+    help_continuing = False
     for line in lines:
         if prefix:
             matched = ITEM_LINE_REGEX.match(line[len(prefix):])
         else:
             matched = ITEM_LINE_REGEX.match(line)
 
+        lstripped_line = line.lstrip()
         if matched:
             if item:
                 # See if we have sub-terms
@@ -72,7 +86,11 @@ def _parse_terms(lines: List[str], prefix: str | None = None) -> Diot:
                     attr = attr.strip()
                     attr_matched = ITEM_ATTR_REGEX.match(attr)
                     if not attr_matched:
-                        raise ValueError(f"Invalid item attribute: {attr}")
+                        raise MalFormattedAnnotationError(
+                            f"\nInvalid item attribute: {attr}"
+                            "\nExpecting: <name>[:=]<value>"
+                            f"\nFull attributes: {attrs}"
+                        )
                     attr_name = attr_matched.group("name")
                     attr_value = attr_matched.group("value")
 
@@ -81,17 +99,27 @@ def _parse_terms(lines: List[str], prefix: str | None = None) -> Diot:
                     )
 
             if help is not None:
-                terms[name].help = help.lstrip()
+                terms[name].help = help.strip()
+                if terms[name].help == "|":
+                    help_continuing = True
 
             item = terms[name]
             just_matched = True
         elif item is None:
-            raise ValueError(f"Invalid item line: {line}")
-        elif just_matched and not line.lstrip().startswith("- "):
-            item.help += f" {line.lstrip()}"
-        elif line.lstrip().startswith("- "):
+            raise MalFormattedAnnotationError(
+                f"\nInvalid item line: {line}"
+                "\nExpecting: <name>[ (<attrs>)]: <help>"
+            )
+        elif just_matched and not lstripped_line.startswith("- "):
+            if help_continuing and item.help == "|":
+                sep = item.help = ""
+            else:
+                sep = "\n" if help_continuing else " "
+            item.help = f"{item.help}{sep}{lstripped_line}"
+        elif lstripped_line.startswith("- "):
             sublines.append(line)
             just_matched = False
+            help_continuing = False
         else:
             sublines.append(line)
 
@@ -152,7 +180,8 @@ def _update_attrs_with_cls(
 
 class Section(ABC):
 
-    def __init__(self, cls) -> None:
+    def __init__(self, cls, name) -> None:
+        self.name = name
         self._cls = cls
         self._lines: List[str] = []
 
@@ -183,10 +212,21 @@ class Section(ABC):
 class SectionSummary(Section):
 
     def parse(self) -> str | Diot | List[str]:
-        return Diot(
-            short=self._lines[0],
-            long="\n".join(self._lines[1:]).lstrip(),
-        )
+        """Parse the summary section."""
+        lines = self._lines
+        if lines[0] and lines[0][0] in (" ", "\t"):
+            lines = _dedent(self._lines)
+        else:
+            lines = [lines[0]] + _dedent(lines[1:])
+
+        short = long = ""
+        for i, line in enumerate(lines):
+            if not line:
+                long = "\n".join(lines[i + 1:])
+                break
+            short += line + " "
+
+        return Diot(short=short.rstrip(), long=long)
 
     @classmethod
     def update_parsed(
@@ -206,7 +246,12 @@ class SectionSummary(Section):
 class SectionItems(Section):
 
     def parse(self) -> str | Diot | List[str]:
-        return _parse_terms(self._lines)
+        try:
+            return _parse_terms(self._lines)
+        except MalFormattedAnnotationError as e:
+            raise MalFormattedAnnotationError(
+                f"[{self._cls.__name__}/{self.name}] {e}"
+            ) from None
 
     @classmethod
     def update_parsed(
@@ -233,7 +278,7 @@ class SectionItems(Section):
 class SectionInput(SectionItems):
 
     def parse(self) -> str | Diot | List[str]:
-        parsed = _parse_terms(self._lines)
+        parsed = super().parse()
         input_keys = self._cls.input
 
         if isinstance(input_keys, str):
@@ -241,11 +286,13 @@ class SectionInput(SectionItems):
                 input_key.strip() for input_key in input_keys.split(",")
             ]
 
+        input_key_names = set()
         for input_key_type in input_keys or []:
             if ":" not in input_key_type:
                 input_key_type = f"{input_key_type}:{ProcInputType.VAR}"
 
             input_key, input_type = input_key_type.split(":", 1)
+            input_key_names.add(input_key)
             if input_key not in parsed:
                 parsed[input_key] = Diot(
                     attrs={"itype": input_type},
@@ -261,6 +308,13 @@ class SectionInput(SectionItems):
             else:
                 parsed[input_key].attrs["action"] = "extend"
 
+        if set(parsed) - input_key_names:
+            warnings.warn(
+                f"[{self._cls.__name__}] Unknown input keys "
+                f"{set(parsed) - input_key_names}",
+                UnknownAnnotationItemWarning,
+            )
+
         return parsed
 
 
@@ -271,16 +325,18 @@ class SectionOutput(SectionItems):
         if not output:
             return Diot()
 
-        parsed = _parse_terms(self._lines)
+        parsed = super().parse()
 
         if not isinstance(output, (list, tuple)):
             output = [out.strip() for out in output.split(",")]
 
+        out_names = set()
         for out in output:
             parts = _parse_one_output(out)
             if not parts:
                 continue
 
+            out_names.add(parts[0])
             if parts[0] not in parsed:
                 parsed[parts[0]] = Diot(
                     attrs={"otype": parts[1], "default": parts[2]},
@@ -291,13 +347,20 @@ class SectionOutput(SectionItems):
                 parsed[parts[0]].attrs["otype"] = parts[1]
                 parsed[parts[0]].attrs["default"] = parts[2]
 
+        if set(parsed) - out_names:
+            warnings.warn(
+                f"[{self._cls.__name__}] Unknown output keys "
+                f"{set(parsed) - out_names}",
+                UnknownAnnotationItemWarning,
+            )
+
         return parsed
 
 
 class SectionEnvs(SectionItems):
 
     def parse(self) -> str | Diot | List[str]:
-        parsed = _parse_terms(self._lines)
+        parsed = super().parse()
         _update_attrs_with_cls(
             parsed,
             self._cls.envs,
